@@ -11,10 +11,10 @@ import os
 import re
 import threading
 import warnings
-from distutils.version import LooseVersion
 
 import numpy as np
 import rasterio
+from packaging import version
 from rasterio.errors import NotGeoreferencedWarning
 from rasterio.vrt import WarpedVRT
 from xarray import Dataset, IndexVariable
@@ -174,14 +174,15 @@ class RasterioArrayWrapper(BackendArray):
         if not np.all(np.asarray(dtypes) == dtypes[0]):
             raise ValueError("All bands should have the same dtype")
 
-        dtype = np.dtype(dtypes[0])
+        dtype = _rasterio_to_numpy_dtype(dtypes)
+
         # handle unsigned case
         if mask_and_scale and unsigned and dtype.kind == "i":
-            self._dtype = np.dtype("u%s" % dtype.itemsize)
+            self._dtype = np.dtype(f"u{dtype.itemsize}")
         elif mask_and_scale and unsigned:
             warnings.warn(
-                "variable %r has _Unsigned attribute but is not "
-                "of integer type. Ignoring attribute." % name,
+                f"variable {name!r} has _Unsigned attribute but is not "
+                "of integer type. Ignoring attribute.",
                 variables.SerializationWarning,
                 stacklevel=3,
             )
@@ -287,11 +288,9 @@ class RasterioArrayWrapper(BackendArray):
                 if self.masked:
                     out = np.ma.filled(out.astype(self.dtype), self.fill_value)
                 if self.mask_and_scale:
-                    for band in np.atleast_1d(band_key):
-                        band_iii = band - 1
-                        out[band_iii] = (
-                            out[band_iii] * riods.scales[band_iii]
-                            + riods.offsets[band_iii]
+                    for iii, band_iii in enumerate(np.atleast_1d(band_key) - 1):
+                        out[iii] = (
+                            out[iii] * riods.scales[band_iii] + riods.offsets[band_iii]
                         )
 
         if squeeze_axis:
@@ -334,6 +333,17 @@ def _parse_envi(meta):
     return parsed_meta
 
 
+def _rasterio_to_numpy_dtype(dtypes):
+    """Numpy dtype from first entry of rasterio dataset.dtypes"""
+    # rasterio has some special dtype names (complex_int16 -> np.complex64)
+    if dtypes[0] == "complex_int16":
+        dtype = np.dtype("complex64")
+    else:
+        dtype = np.dtype(dtypes[0])
+
+    return dtype
+
+
 def _to_numeric(value):
     """
     Convert the value to a number
@@ -487,36 +497,34 @@ def _get_rasterio_attrs(riods):
     # pylint: disable=too-many-branches
     # Add rasterio attributes
     attrs = _parse_tags(riods.tags(1))
-    if hasattr(riods, "nodata") and riods.nodata is not None:
+    if riods.nodata is not None:
         # The nodata values for the raster bands
         attrs["_FillValue"] = riods.nodata
-    if hasattr(riods, "scales"):
-        # The scale values for the raster bands
-        if len(set(riods.scales)) > 1:
-            attrs["scales"] = riods.scales
-            warnings.warn(
-                "Offsets differ across bands. The 'scale_factor' attribute will "
-                "not be added. See the 'scales' attribute."
-            )
-        else:
-            attrs["scale_factor"] = riods.scales[0]
-    if hasattr(riods, "offsets"):
-        # The offset values for the raster bands
-        if len(set(riods.offsets)) > 1:
-            attrs["offsets"] = riods.offsets
-            warnings.warn(
-                "Offsets differ across bands. The 'add_offset' attribute will "
-                "not be added. See the 'offsets' attribute."
-            )
-        else:
-            attrs["add_offset"] = riods.offsets[0]
-    if hasattr(riods, "descriptions") and any(riods.descriptions):
+    # The scale values for the raster bands
+    if len(set(riods.scales)) > 1:
+        attrs["scales"] = riods.scales
+        warnings.warn(
+            "Offsets differ across bands. The 'scale_factor' attribute will "
+            "not be added. See the 'scales' attribute."
+        )
+    else:
+        attrs["scale_factor"] = riods.scales[0]
+    # The offset values for the raster bands
+    if len(set(riods.offsets)) > 1:
+        attrs["offsets"] = riods.offsets
+        warnings.warn(
+            "Offsets differ across bands. The 'add_offset' attribute will "
+            "not be added. See the 'offsets' attribute."
+        )
+    else:
+        attrs["add_offset"] = riods.offsets[0]
+    if any(riods.descriptions):
         if len(set(riods.descriptions)) == 1:
             attrs["long_name"] = riods.descriptions[0]
         else:
             # Descriptions for each dataset band
             attrs["long_name"] = riods.descriptions
-    if hasattr(riods, "units") and any(riods.units):
+    if any(riods.units):
         # A list of units string for each dataset band
         if len(riods.units) == 1:
             attrs["units"] = riods.units[0]
@@ -654,10 +662,10 @@ def _prepare_dask(result, riods, filename, chunks):
         import dask
         from dask.array.core import normalize_chunks
 
-        if LooseVersion(dask.__version__) < LooseVersion("0.18.0"):
+        if version.parse(dask.__version__) < version.parse("0.18.0"):
             msg = (
                 "Automatic chunking requires dask.__version__ >= 0.18.0 . "
-                "You currently have version %s" % dask.__version__
+                f"You currently have version {dask.__version__}"
             )
             raise NotImplementedError(msg)
         block_shape = (1,) + riods.block_shapes[0]
@@ -668,7 +676,7 @@ def _prepare_dask(result, riods, filename, chunks):
             previous_chunks=tuple((c,) for c in block_shape),
         )
     token = tokenize(filename, mtime, chunks)
-    name_prefix = "open_rasterio-%s" % token
+    name_prefix = f"open_rasterio-{token}"
     return result.chunk(chunks, name_prefix=name_prefix, token=token)
 
 
@@ -790,8 +798,8 @@ def open_rasterio(
         vrt = filename
         filename = vrt.src_dataset.name
         vrt_params = dict(
-            src_crs=vrt.src_crs.to_string(),
-            crs=vrt.crs.to_string(),
+            src_crs=vrt.src_crs.to_string() if vrt.src_crs else None,
+            crs=vrt.crs.to_string() if vrt.crs else None,
             resampling=vrt.resampling,
             tolerance=vrt.tolerance,
             src_nodata=vrt.src_nodata,
@@ -823,8 +831,6 @@ def open_rasterio(
         riods = manager.acquire()
         captured_warnings = rio_warnings.copy()
 
-    riods = manager.acquire()
-    captured_warnings = rio_warnings.copy()
     # raise the NotGeoreferencedWarning if applicable
     for rio_warning in captured_warnings:
         if not riods.subdatasets or not isinstance(
@@ -872,6 +878,10 @@ def open_rasterio(
         coord_name = "band"
         coords[coord_name] = np.asarray(riods.indexes)
 
+    has_gcps = riods.gcps[0]
+    if has_gcps:
+        parse_coordinates = False
+
     # Get geospatial coordinates
     if parse_coordinates:
         coords.update(
@@ -882,8 +892,9 @@ def open_rasterio(
     encoding = {}
     if mask_and_scale and "_Unsigned" in attrs:
         unsigned = variables.pop_to(attrs, encoding, "_Unsigned") == "true"
+
     if masked:
-        encoding["dtype"] = str(riods.dtypes[0])
+        encoding["dtype"] = str(_rasterio_to_numpy_dtype(riods.dtypes))
 
     da_name = attrs.pop("NETCDF_VARNAME", default_name)
     data = indexing.LazilyOuterIndexedArray(
@@ -926,8 +937,10 @@ def open_rasterio(
     # always (0, 0, 1) per definition (see
     # https://github.com/sgillies/affine)
     result.rio.write_transform(_rio_transform(riods), inplace=True)
-    if hasattr(riods, "crs") and riods.crs:
+    if riods.crs:
         result.rio.write_crs(riods.crs, inplace=True)
+    if has_gcps:
+        result.rio.write_gcps(*riods.gcps, inplace=True)
 
     if chunks is not None:
         result = _prepare_dask(result, riods, filename, chunks)
@@ -937,4 +950,5 @@ def open_rasterio(
     result.rio._manager = manager
     # add file path to encoding
     result.encoding["source"] = riods.name
+    result.encoding["rasterio_dtype"] = str(riods.dtypes[0])
     return result
