@@ -10,8 +10,6 @@ datacube is licensed under the Apache License, Version 2.0:
 
 """
 import copy
-import warnings
-from distutils.version import LooseVersion
 from typing import Iterable
 
 import numpy as np
@@ -19,9 +17,10 @@ import rasterio
 import rasterio.mask
 import rasterio.warp
 import xarray
+from packaging import version
+from rasterio.dtypes import dtype_rev
 from rasterio.enums import Resampling
 from rasterio.features import geometry_mask
-from scipy.interpolate import griddata
 from xarray.core.dtypes import get_fill_value
 
 from rioxarray.crs import crs_from_user_input
@@ -31,9 +30,32 @@ from rioxarray.exceptions import (
     OneDimensionalRaster,
     RioXarrayError,
 )
-from rioxarray.raster_writer import FILL_VALUE_NAMES, UNWANTED_RIO_ATTRS, RasterioWriter
+from rioxarray.raster_writer import (
+    FILL_VALUE_NAMES,
+    UNWANTED_RIO_ATTRS,
+    RasterioWriter,
+    _ensure_nodata_dtype,
+)
 from rioxarray.rioxarray import XRasterBase, _get_data_var_message, _make_coords
 
+# DTYPE TO NODATA MAP
+# Based on: https://github.com/OSGeo/gdal/blob/
+# cde27dc7641964a872efdc6bbcf5e3d3f7ab9cfd/gdal/
+# swig/python/gdal-utils/osgeo_utils/gdal_calc.py#L62
+_NODATA_DTYPE_MAP = {
+    1: 255,  # GDT_Byte
+    2: 65535,  # GDT_UInt16
+    3: -32768,  # GDT_Int16
+    4: 4294967293,  # GDT_UInt32
+    5: -2147483647,  # GDT_Int32
+    6: 3.402823466e38,  # GDT_Float32
+    7: 1.7976931348623158e308,  # GDT_Float64
+    8: -32768,  # GDT_CInt16
+    9: -2147483647,  # GDT_CInt32
+    10: 3.402823466e38,  # GDT_CFloat32
+    11: 1.7976931348623158e308,  # GDT_CFloat64
+}
+
 
 def _generate_attrs(src_data_array, dst_nodata):
     # add original attributes
@@ -83,10 +105,10 @@ def _add_attrs_proj(new_data_array, src_data_array):
 
 
 def _make_dst_affine(
-    src_data_array, src_crs, dst_crs, dst_resolution=None, dst_shape=None
+    src_data_array, src_crs, dst_crs, dst_resolution=None, dst_shape=None, **kwargs
 ):
     """Determine the affine of the new projected `xarray.DataArray`"""
-    src_bounds = src_data_array.rio.bounds()
+    src_bounds = () if "gcps" in kwargs else src_data_array.rio.bounds()
     src_height, src_width = src_data_array.rio.shape
     dst_height, dst_width = dst_shape if dst_shape is not None else (None, None)
     # pylint: disable=isinstance-second-argument-not-valid-type
@@ -94,41 +116,25 @@ def _make_dst_affine(
         dst_resolution = tuple(abs(res_val) for res_val in dst_resolution)
     elif dst_resolution is not None:
         dst_resolution = abs(dst_resolution)
-    resolution_or_width_height = {
-        k: v
-        for k, v in [
-            ("resolution", dst_resolution),
-            ("dst_height", dst_height),
-            ("dst_width", dst_width),
-        ]
-        if v is not None
-    }
+
+    for key, value in (
+        ("resolution", dst_resolution),
+        ("dst_height", dst_height),
+        ("dst_width", dst_width),
+    ):
+        if value is not None:
+            kwargs[key] = value
     dst_affine, dst_width, dst_height = rasterio.warp.calculate_default_transform(
         src_crs,
         dst_crs,
         src_width,
         src_height,
         *src_bounds,
-        **resolution_or_width_height,
+        **kwargs,
     )
     return dst_affine, dst_width, dst_height
 
 
-def _ensure_nodata_dtype(original_nodata, new_dtype):
-    """
-    Convert the nodata to the new datatype and raise warning
-    if the value of the nodata value changed.
-    """
-    original_nodata = float(original_nodata)
-    nodata = np.dtype(new_dtype).type(original_nodata)
-    if not np.isnan(nodata) and original_nodata != nodata:
-        warnings.warn(
-            f"The nodata value ({original_nodata}) has been automatically "
-            f"changed to ({nodata}) to match the dtype of the data."
-        )
-    return nodata
-
-
 def _clip_from_disk(xds, geometries, all_touched, drop, invert):
     """
     clip from disk if the file object is available
@@ -330,6 +336,8 @@ class RasterArray(XRasterBase):
         shape=None,
         transform=None,
         resampling=Resampling.nearest,
+        nodata=None,
+        **kwargs,
     ):
         """
         Reproject :obj:`xarray.DataArray` objects
@@ -343,6 +351,7 @@ class RasterArray(XRasterBase):
 
         .. versionadded:: 0.0.27 shape
         .. versionadded:: 0.0.28 transform
+        .. versionadded:: 0.5.0 nodata, kwargs
 
         Parameters
         ----------
@@ -354,10 +363,21 @@ class RasterArray(XRasterBase):
         shape: tuple(int, int), optional
             Shape of the destination in pixels (dst_height, dst_width). Cannot be used
             together with resolution.
-        transform: optional
+        transform: Affine, optional
             The destination transform.
         resampling: rasterio.enums.Resampling, optional
             See :func:`rasterio.warp.reproject` for more details.
+        nodata: float, optional
+            The nodata value used to initialize the destination;
+            it will remain in all areas not covered by the reprojected source.
+            Defaults to the nodata value of the source image if none provided
+            and exists or attempts to find an appropriate value by dtype.
+        **kwargs: dict
+            Additional keyword arguments to pass into :func:`rasterio.warp.reproject`.
+            To override:
+            - src_transform: `rio.write_transform`
+            - src_crs: `rio.write_crs`
+            - src_nodata: `rio.write_nodata`
 
 
         Returns
@@ -372,10 +392,14 @@ class RasterArray(XRasterBase):
                 "CRS not found. Please set the CRS with 'rio.write_crs()'."
                 f"{_get_data_var_message(self._obj)}"
             )
-        src_affine = self.transform(recalc=True)
+        gcps = self.get_gcps()
+        if gcps:
+            kwargs.setdefault("gcps", gcps)
+
+        src_affine = None if "gcps" in kwargs else self.transform(recalc=True)
         if transform is None:
             dst_affine, dst_width, dst_height = _make_dst_affine(
-                self._obj, self.crs, dst_crs, resolution, shape
+                self._obj, self.crs, dst_crs, resolution, shape, **kwargs
             )
         else:
             dst_affine = transform
@@ -384,31 +408,21 @@ class RasterArray(XRasterBase):
             else:
                 dst_height, dst_width = self.shape
 
-        extra_dim = self._check_dimensions()
-        if extra_dim:
-            dst_data = np.zeros(
-                (self._obj[extra_dim].size, dst_height, dst_width),
-                dtype=self._obj.dtype.type,
-            )
-        else:
-            dst_data = np.zeros((dst_height, dst_width), dtype=self._obj.dtype.type)
+        dst_data = self._create_dst_data(dst_height, dst_width)
+
+        dst_nodata = self._get_dst_nodata(nodata)
 
-        dst_nodata = self._obj.dtype.type(
-            self.nodata if self.nodata is not None else -9999
-        )
-        src_nodata = self._obj.dtype.type(
-            self.nodata if self.nodata is not None else dst_nodata
-        )
         rasterio.warp.reproject(
             source=self._obj.values,
             destination=dst_data,
             src_transform=src_affine,
             src_crs=self.crs,
-            src_nodata=src_nodata,
+            src_nodata=self.nodata,
             dst_transform=dst_affine,
             dst_crs=dst_crs,
             dst_nodata=dst_nodata,
             resampling=resampling,
+            **kwargs,
         )
         # add necessary attributes
         new_attrs = _generate_attrs(self._obj, dst_nodata)
@@ -434,7 +448,29 @@ class RasterArray(XRasterBase):
         xda.rio.write_coordinate_system(inplace=True)
         return xda
 
-    def reproject_match(self, match_data_array, resampling=Resampling.nearest):
+    def _get_dst_nodata(self, nodata):
+        default_nodata = (
+            _NODATA_DTYPE_MAP[dtype_rev[self._obj.dtype.name]]
+            if self.nodata is None
+            else self.nodata
+        )
+        dst_nodata = default_nodata if nodata is None else nodata
+        return dst_nodata
+
+    def _create_dst_data(self, dst_height, dst_width):
+        extra_dim = self._check_dimensions()
+        if extra_dim:
+            dst_data = np.zeros(
+                (self._obj[extra_dim].size, dst_height, dst_width),
+                dtype=self._obj.dtype.type,
+            )
+        else:
+            dst_data = np.zeros((dst_height, dst_width), dtype=self._obj.dtype.type)
+        return dst_data
+
+    def reproject_match(
+        self, match_data_array, resampling=Resampling.nearest, **reproject_kwargs
+    ):
         """
         Reproject a DataArray object to match the resolution, projection,
         and region of another DataArray.
@@ -446,13 +482,16 @@ class RasterArray(XRasterBase):
             a 'crs' attribute to be set containing a valid CRS.
             If using a WKT (e.g. from spatiareference.org), make sure it is an OGC WKT.
 
+        .. versionadded:: 0.9 reproject_kwargs
+
         Parameters
         ----------
         match_data_array:  :obj:`xarray.DataArray` | :obj:`xarray.Dataset`
             DataArray of the target resolution and projection.
         resampling: rasterio.enums.Resampling, optional
             See :func:`rasterio.warp.reproject` for more details.
-
+        **reproject_kwargs:
+            Other options to pass to :meth:`rioxarray.raster_array.RasterArray.reproject`
 
         Returns
         --------
@@ -460,12 +499,33 @@ class RasterArray(XRasterBase):
             Contains the data from the src_data_array, reprojected to match
             match_data_array.
         """
-        return self.reproject(
+        reprojected_data_array = self.reproject(
             match_data_array.rio.crs,
             transform=match_data_array.rio.transform(recalc=True),
             shape=match_data_array.rio.shape,
             resampling=resampling,
+            **reproject_kwargs,
         )
+        # hack to resolve: https://github.com/corteva/rioxarray/issues/298
+        # may be resolved in the future by flexible indexes:
+        # https://github.com/pydata/xarray/pull/4489#issuecomment-831809607
+        x_attrs = reprojected_data_array[reprojected_data_array.rio.x_dim].attrs.copy()
+        y_attrs = reprojected_data_array[reprojected_data_array.rio.y_dim].attrs.copy()
+        # ensure coords the same
+        reprojected_data_array = reprojected_data_array.assign_coords(
+            {
+                reprojected_data_array.rio.x_dim: match_data_array[
+                    match_data_array.rio.x_dim
+                ].values.copy(),
+                reprojected_data_array.rio.y_dim: match_data_array[
+                    match_data_array.rio.y_dim
+                ].values.copy(),
+            }
+        )
+        # ensure attributes copied
+        reprojected_data_array[reprojected_data_array.rio.x_dim].attrs = x_attrs
+        reprojected_data_array[reprojected_data_array.rio.y_dim].attrs = y_attrs
+        return reprojected_data_array
 
     def pad_xy(self, minx, miny, maxx, maxy, constant_values):
         """Pad the array to x,y bounds.
@@ -641,7 +701,6 @@ class RasterArray(XRasterBase):
         # check that the window has data in it
         if window_error or cl_array.rio.width <= 1 or cl_array.rio.height <= 1:
             if auto_expand and auto_expand < auto_expand_limit:
-                resolution_x, resolution_y = self.resolution()
                 return self.clip_box(
                     minx=minx - abs(resolution_x) / 2.0,
                     miny=miny - abs(resolution_y) / 2.0,
@@ -734,7 +793,7 @@ class RasterArray(XRasterBase):
             )
         crs = crs_from_user_input(crs) if crs is not None else self.crs
         if self.crs != crs:
-            if LooseVersion(rasterio.__version__) >= LooseVersion("1.2"):
+            if version.parse(rasterio.__version__) >= version.parse("1.2"):
                 geometries = rasterio.warp.transform_geom(crs, self.crs, geometries)
             else:
                 geometries = [
@@ -786,6 +845,15 @@ class RasterArray(XRasterBase):
         :class:`numpy.ndarray`:
             An interpolated :class:`numpy.ndarray`.
         """
+        try:
+            from scipy.interpolate import (  # pylint: disable=import-outside-toplevel,import-error
+                griddata,
+            )
+        except ModuleNotFoundError as err:
+            raise ModuleNotFoundError(
+                "scipy is not found. Use rioxarray[interp] to install."
+            ) from err
+
         src_data_flat = src_data.flatten()
         try:
             data_isnan = np.isnan(self.nodata)
@@ -815,6 +883,8 @@ class RasterArray(XRasterBase):
         """
         This method uses scipy.interpolate.griddata to interpolate missing data.
 
+        .. warning:: scipy is an optional dependency.
+
         Parameters
         ----------
         method: {‘linear’, ‘nearest’, ‘cubic’}, optional
@@ -910,14 +980,11 @@ class RasterArray(XRasterBase):
             is True. Otherwise None is returned.
 
         """
-        if driver is None and LooseVersion(rasterio.__version__) < LooseVersion("1.2"):
+        if driver is None and version.parse(rasterio.__version__) < version.parse(
+            "1.2"
+        ):
             driver = "GTiff"
 
-        dtype = (
-            self._obj.encoding.get("dtype", str(self._obj.dtype))
-            if dtype is None
-            else dtype
-        )
         # get the output profile from the rasterio object
         # if opened with xarray.open_rasterio()
         try:
@@ -945,11 +1012,6 @@ class RasterArray(XRasterBase):
         rio_nodata = (
             self.encoded_nodata if self.encoded_nodata is not None else self.nodata
         )
-        if rio_nodata is not None:
-            # Ensure dtype of output data matches the expected dtype.
-            # This check is added here as the dtype of the data is
-            # converted right before writing.
-            rio_nodata = _ensure_nodata_dtype(rio_nodata, dtype)
 
         return RasterioWriter(raster_path=raster_path).to_raster(
             xarray_dataarray=self._obj,
@@ -961,6 +1023,7 @@ class RasterArray(XRasterBase):
             dtype=dtype,
             crs=self.crs,
             transform=self.transform(recalc=recalc_transform),
+            gcps=self.get_gcps(),
             nodata=rio_nodata,
             windowed=windowed,
             lock=lock,
