@@ -8,8 +8,9 @@ import numpy as np
 import xarray
 from rasterio.enums import Resampling
 
-from rioxarray.exceptions import RioXarrayError
-from rioxarray.rioxarray import XRasterBase
+from rioxarray._options import SKIP_MISSING_SPATIAL_DIMS, get_option
+from rioxarray.exceptions import MissingSpatialDimensionError, RioXarrayError
+from rioxarray.rioxarray import XRasterBase, _get_spatial_dims
 
 
 @xarray.register_dataset_accessor("rio")
@@ -46,9 +47,7 @@ class RasterDataset(XRasterBase):
         if all(crs_i == crs for crs_i in crs_list):
             self._crs = crs
         else:
-            raise RioXarrayError(
-                "CRS in DataArrays differ in the Dataset: {}".format(crs_list)
-            )
+            raise RioXarrayError(f"CRS in DataArrays differ in the Dataset: {crs_list}")
         return self._crs
 
     def reproject(
@@ -58,17 +57,21 @@ class RasterDataset(XRasterBase):
         shape=None,
         transform=None,
         resampling=Resampling.nearest,
+        nodata=None,
+        **kwargs,
     ):
         """
         Reproject :class:`xarray.Dataset` objects
 
         .. note:: Only 2D/3D arrays with dimensions 'x'/'y' are currently supported.
+            Others are appended as is.
             Requires either a grid mapping variable with 'spatial_ref' or
             a 'crs' attribute to be set containing a valid CRS.
             If using a WKT (e.g. from spatiareference.org), make sure it is an OGC WKT.
 
         .. versionadded:: 0.0.27 shape
         .. versionadded:: 0.0.28 transform
+        .. versionadded:: 0.5.0 nodata, kwargs
 
         Parameters
         ----------
@@ -84,7 +87,17 @@ class RasterDataset(XRasterBase):
             The destination transform.
         resampling: rasterio.enums.Resampling, optional
             See :func:`rasterio.warp.reproject` for more details.
-
+        nodata: float, optional
+            The nodata value used to initialize the destination;
+            it will remain in all areas not covered by the reprojected source.
+            Defaults to the nodata value of the source image if none provided
+            and exists or attempts to find an appropriate value by dtype.
+        **kwargs: dict
+            Additional keyword arguments to pass into :func:`rasterio.warp.reproject`.
+            To override:
+            - src_transform: `rio.write_transform`
+            - src_crs: `rio.write_crs`
+            - src_nodata: `rio.write_nodata`
 
         Returns
         --------
@@ -93,29 +106,43 @@ class RasterDataset(XRasterBase):
         """
         resampled_dataset = xarray.Dataset(attrs=self._obj.attrs)
         for var in self.vars:
-            resampled_dataset[var] = (
-                self._obj[var]
-                .rio.set_spatial_dims(x_dim=self.x_dim, y_dim=self.y_dim, inplace=True)
-                .rio.reproject(
-                    dst_crs,
-                    resolution=resolution,
-                    shape=shape,
-                    transform=transform,
-                    resampling=resampling,
+            try:
+                x_dim, y_dim = _get_spatial_dims(self._obj, var)
+                resampled_dataset[var] = (
+                    self._obj[var]
+                    .rio.set_spatial_dims(x_dim=x_dim, y_dim=y_dim, inplace=True)
+                    .rio.reproject(
+                        dst_crs,
+                        resolution=resolution,
+                        shape=shape,
+                        transform=transform,
+                        resampling=resampling,
+                        nodata=nodata,
+                        **kwargs,
+                    )
                 )
-            )
+            except MissingSpatialDimensionError:
+                if len(self._obj[var].dims) >= 2 and not get_option(
+                    SKIP_MISSING_SPATIAL_DIMS
+                ):
+                    raise
+                resampled_dataset[var] = self._obj[var].copy()
         return resampled_dataset
 
-    def reproject_match(self, match_data_array, resampling=Resampling.nearest):
+    def reproject_match(
+        self, match_data_array, resampling=Resampling.nearest, **reproject_kwargs
+    ):
         """
         Reproject a Dataset object to match the resolution, projection,
         and region of another DataArray.
 
         .. note:: Only 2D/3D arrays with dimensions 'x'/'y' are currently supported.
+            Others are appended as is.
             Requires either a grid mapping variable with 'spatial_ref' or
             a 'crs' attribute to be set containing a valid CRS.
             If using a WKT (e.g. from spatiareference.org), make sure it is an OGC WKT.
 
+        .. versionadded:: 0.9 reproject_kwargs
 
         Parameters
         ----------
@@ -123,7 +150,8 @@ class RasterDataset(XRasterBase):
             Dataset with the target resolution and projection.
         resampling: rasterio.enums.Resampling, optional
             See :func:`rasterio.warp.reproject` for more details.
-
+        **reproject_kwargs:
+            Other options to pass to :meth:`rioxarray.raster_dataset.RasterDataset.reproject`
 
         Returns
         --------
@@ -133,11 +161,21 @@ class RasterDataset(XRasterBase):
         """
         resampled_dataset = xarray.Dataset(attrs=self._obj.attrs)
         for var in self.vars:
-            resampled_dataset[var] = (
-                self._obj[var]
-                .rio.set_spatial_dims(x_dim=self.x_dim, y_dim=self.y_dim, inplace=True)
-                .rio.reproject_match(match_data_array, resampling=resampling)
-            )
+            try:
+                x_dim, y_dim = _get_spatial_dims(self._obj, var)
+                resampled_dataset[var] = (
+                    self._obj[var]
+                    .rio.set_spatial_dims(x_dim=x_dim, y_dim=y_dim, inplace=True)
+                    .rio.reproject_match(
+                        match_data_array, resampling=resampling, **reproject_kwargs
+                    )
+                )
+            except MissingSpatialDimensionError:
+                if len(self._obj[var].dims) >= 2 and not get_option(
+                    SKIP_MISSING_SPATIAL_DIMS
+                ):
+                    raise
+                resampled_dataset[var] = self._obj[var].copy()
         return resampled_dataset.rio.set_spatial_dims(
             x_dim=self.x_dim, y_dim=self.y_dim, inplace=True
         )
@@ -148,6 +186,8 @@ class RasterDataset(XRasterBase):
         .. warning:: Only works if all variables in the dataset have the
                      same coordinates.
 
+        .. warning:: Pads variables that have dimensions 'x'/'y'. Others are appended as is.
+
         Parameters
         ----------
         minx: float
@@ -166,20 +206,27 @@ class RasterDataset(XRasterBase):
         """
         padded_dataset = xarray.Dataset(attrs=self._obj.attrs)
         for var in self.vars:
-            padded_dataset[var] = (
-                self._obj[var]
-                .rio.set_spatial_dims(x_dim=self.x_dim, y_dim=self.y_dim, inplace=True)
-                .rio.pad_box(minx, miny, maxx, maxy)
-            )
+            try:
+                x_dim, y_dim = _get_spatial_dims(self._obj, var)
+                padded_dataset[var] = (
+                    self._obj[var]
+                    .rio.set_spatial_dims(x_dim=x_dim, y_dim=y_dim, inplace=True)
+                    .rio.pad_box(minx, miny, maxx, maxy)
+                )
+            except MissingSpatialDimensionError:
+                if len(self._obj[var].dims) >= 2 and not get_option(
+                    SKIP_MISSING_SPATIAL_DIMS
+                ):
+                    raise
+                padded_dataset[var] = self._obj[var].copy()
         return padded_dataset.rio.set_spatial_dims(
             x_dim=self.x_dim, y_dim=self.y_dim, inplace=True
         )
 
     def clip_box(self, minx, miny, maxx, maxy, auto_expand=False, auto_expand_limit=3):
-        """Clip the :class:`xarray.Dataset` by a bounding box.
+        """Clip the :class:`xarray.Dataset` by a bounding box in dimensions 'x'/'y'.
 
-        .. warning:: Only works if all variables in the dataset have the
-                     same coordinates.
+        .. warning:: Clips variables that have dimensions 'x'/'y'. Others are appended as is.
 
         Parameters
         ----------
@@ -204,18 +251,26 @@ class RasterDataset(XRasterBase):
         """
         clipped_dataset = xarray.Dataset(attrs=self._obj.attrs)
         for var in self.vars:
-            clipped_dataset[var] = (
-                self._obj[var]
-                .rio.set_spatial_dims(x_dim=self.x_dim, y_dim=self.y_dim, inplace=True)
-                .rio.clip_box(
-                    minx,
-                    miny,
-                    maxx,
-                    maxy,
-                    auto_expand=auto_expand,
-                    auto_expand_limit=auto_expand_limit,
+            try:
+                x_dim, y_dim = _get_spatial_dims(self._obj, var)
+                clipped_dataset[var] = (
+                    self._obj[var]
+                    .rio.set_spatial_dims(x_dim=x_dim, y_dim=y_dim, inplace=True)
+                    .rio.clip_box(
+                        minx,
+                        miny,
+                        maxx,
+                        maxy,
+                        auto_expand=auto_expand,
+                        auto_expand_limit=auto_expand_limit,
+                    )
                 )
-            )
+            except MissingSpatialDimensionError:
+                if len(self._obj[var].dims) >= 2 and not get_option(
+                    SKIP_MISSING_SPATIAL_DIMS
+                ):
+                    raise
+                clipped_dataset[var] = self._obj[var].copy()
         return clipped_dataset.rio.set_spatial_dims(
             x_dim=self.x_dim, y_dim=self.y_dim, inplace=True
         )
@@ -230,10 +285,9 @@ class RasterDataset(XRasterBase):
         from_disk=False,
     ):
         """
-        Crops a :class:`xarray.Dataset` by geojson like geometry dicts.
+        Crops a :class:`xarray.Dataset` by geojson like geometry dicts in dimensions 'x'/'y'.
 
-        .. warning:: Only works if all variables in the dataset have the same
-                     coordinates.
+        .. warning:: Clips variables that have dimensions 'x'/'y'. Others are appended as is.
 
         Powered by `rasterio.features.geometry_mask`.
 
@@ -265,7 +319,7 @@ class RasterDataset(XRasterBase):
             false, only pixels whose center is within the polygon or that
             are selected by Bresenham's line algorithm will be burned in.
         drop: bool, optional
-            If True, drop the data outside of the extent of the mask geoemtries
+            If True, drop the data outside of the extent of the mask geometries
             Otherwise, it will return the same raster with the data masked.
             Default is True.
         invert: boolean, optional
@@ -284,18 +338,26 @@ class RasterDataset(XRasterBase):
         """
         clipped_dataset = xarray.Dataset(attrs=self._obj.attrs)
         for var in self.vars:
-            clipped_dataset[var] = (
-                self._obj[var]
-                .rio.set_spatial_dims(x_dim=self.x_dim, y_dim=self.y_dim, inplace=True)
-                .rio.clip(
-                    geometries,
-                    crs=crs,
-                    all_touched=all_touched,
-                    drop=drop,
-                    invert=invert,
-                    from_disk=from_disk,
+            try:
+                x_dim, y_dim = _get_spatial_dims(self._obj, var)
+                clipped_dataset[var] = (
+                    self._obj[var]
+                    .rio.set_spatial_dims(x_dim=x_dim, y_dim=y_dim, inplace=True)
+                    .rio.clip(
+                        geometries,
+                        crs=crs,
+                        all_touched=all_touched,
+                        drop=drop,
+                        invert=invert,
+                        from_disk=from_disk,
+                    )
                 )
-            )
+            except MissingSpatialDimensionError:
+                if len(self._obj[var].dims) >= 2 and not get_option(
+                    SKIP_MISSING_SPATIAL_DIMS
+                ):
+                    raise
+                clipped_dataset[var] = self._obj[var].copy()
         return clipped_dataset.rio.set_spatial_dims(
             x_dim=self.x_dim, y_dim=self.y_dim, inplace=True
         )
@@ -304,6 +366,10 @@ class RasterDataset(XRasterBase):
         """
         This method uses `scipy.interpolate.griddata` to interpolate missing data.
 
+        .. warning:: scipy is an optional dependency.
+
+        .. warning:: Interpolates variables that have dimensions 'x'/'y'. Others are appended as is.
+
         Parameters
         ----------
         method: {‘linear’, ‘nearest’, ‘cubic’}, optional
@@ -316,11 +382,19 @@ class RasterDataset(XRasterBase):
         """
         interpolated_dataset = xarray.Dataset(attrs=self._obj.attrs)
         for var in self.vars:
-            interpolated_dataset[var] = (
-                self._obj[var]
-                .rio.set_spatial_dims(x_dim=self.x_dim, y_dim=self.y_dim, inplace=True)
-                .rio.interpolate_na(method=method)
-            )
+            try:
+                x_dim, y_dim = _get_spatial_dims(self._obj, var)
+                interpolated_dataset[var] = (
+                    self._obj[var]
+                    .rio.set_spatial_dims(x_dim=x_dim, y_dim=y_dim, inplace=True)
+                    .rio.interpolate_na(method=method)
+                )
+            except MissingSpatialDimensionError:
+                if len(self._obj[var].dims) >= 2 and not get_option(
+                    SKIP_MISSING_SPATIAL_DIMS
+                ):
+                    raise
+                interpolated_dataset[var] = self._obj[var].copy()
         return interpolated_dataset.rio.set_spatial_dims(
             x_dim=self.x_dim, y_dim=self.y_dim, inplace=True
         )
@@ -379,7 +453,7 @@ class RasterDataset(XRasterBase):
             is True. Otherwise None is returned.
 
         """
-        variable_dim = "band_{}".format(uuid4())
+        variable_dim = f"band_{uuid4()}"
         data_array = self._obj.to_array(dim=variable_dim)
         # write data array names to raster
         data_array.attrs["long_name"] = data_array[variable_dim].values.tolist()
@@ -402,12 +476,16 @@ class RasterDataset(XRasterBase):
         else:
             raise RioXarrayError(
                 "All nodata values must be the same when exporting to raster. "
-                "Current values: {}".format(nodatavals)
+                f"Current values: {nodatavals}"
             )
         if self.crs is not None:
             data_array.rio.write_crs(self.crs, inplace=True)
         # write it to a raster
-        return data_array.rio.to_raster(
+        return data_array.rio.set_spatial_dims(
+            x_dim=self.x_dim,
+            y_dim=self.y_dim,
+            inplace=True,
+        ).rio.to_raster(
             raster_path=raster_path,
             driver=driver,
             dtype=dtype,
