@@ -12,6 +12,7 @@ import rasterio.warp
 import rasterio.windows
 import xarray
 from affine import Affine
+from rasterio.control import GroundControlPoint
 from rasterio.crs import CRS
 
 from rioxarray._options import EXPORT_GRID_MAPPING, get_option
@@ -21,6 +22,7 @@ from rioxarray.exceptions import (
     DimensionMissingCoordinateError,
     InvalidDimensionOrder,
     MissingCRS,
+    MissingSpatialDimensionError,
     NoDataInBounds,
     OneDimensionalRaster,
     RioXarrayError,
@@ -30,6 +32,45 @@ from rioxarray.exceptions import (
 DEFAULT_GRID_MAP = "spatial_ref"
 
 
+def _affine_has_rotation(affine) -> bool:
+    """
+    Determine if the affine has rotation.
+
+    Parameters
+    ----------
+    affine: :obj:`affine.Affine`
+        The affine of the grid.
+
+    Returns
+    -------
+    bool
+    """
+    return affine.b == affine.d != 0
+
+
+def _resolution(affine):
+    """
+    Determine if the resolution of the affine.
+    If it has rotation, the sign of the resolution is lost.
+
+    Based on: https://github.com/mapbox/rasterio/blob/6185a4e4ad72b5669066d2d5004bf46d94a6d298/rasterio/_base.pyx#L943-L951
+
+    Parameters
+    ----------
+    affine: :obj:`affine.Affine`
+        The affine of the grid.
+
+    x_resolution, y_resolution: float
+        The resolution of the affine.
+    """
+    if not _affine_has_rotation(affine):
+        return affine.a, affine.e
+    return (
+        math.sqrt(affine.a**2 + affine.d**2),
+        math.sqrt(affine.b**2 + affine.e**2),
+    )
+
+
 def affine_to_coords(affine, width, height, x_dim="x", y_dim="y"):
     """Generate 1d pixel centered coordinates from affine.
 
@@ -53,13 +94,14 @@ def affine_to_coords(affine, width, height, x_dim="x", y_dim="y"):
     dict: x and y coordinate arrays.
 
     """
-    if affine.is_rectilinear:
-        x_coords, _ = affine * (np.arange(width) + 0.5, np.zeros(width) + 0.5)
-        _, y_coords = affine * (np.zeros(height) + 0.5, np.arange(height) + 0.5)
+    transform = affine * affine.translation(0.5, 0.5)
+    if affine.is_rectilinear and not _affine_has_rotation(affine):
+        x_coords, _ = transform * (np.arange(width), np.zeros(width))
+        _, y_coords = transform * (np.zeros(height), np.arange(height))
     else:
-        x_coords, y_coords = affine * np.meshgrid(
-            np.arange(width) + 0.5,
-            np.arange(height) + 0.5,
+        x_coords, y_coords = transform * np.meshgrid(
+            np.arange(width),
+            np.arange(height),
         )
     return {y_dim: y_coords, x_dim: x_coords}
 
@@ -67,7 +109,7 @@ def affine_to_coords(affine, width, height, x_dim="x", y_dim="y"):
 def _generate_spatial_coords(affine, width, height):
     """get spatial coords in new transform"""
     new_spatial_coords = affine_to_coords(affine, width, height)
-    if affine.is_rectilinear:
+    if new_spatial_coords["x"].ndim == 1:
         return {
             "x": xarray.IndexVariable("x", new_spatial_coords["x"]),
             "y": xarray.IndexVariable("y", new_spatial_coords["y"]),
@@ -125,6 +167,34 @@ def _get_data_var_message(obj):
         return ""
 
 
+def _get_spatial_dims(obj: xarray.Dataset, var: str):
+    """
+    Retrieve the spatial dimensions of the dataset
+    """
+    try:
+        return obj[var].rio.x_dim, obj[var].rio.y_dim
+    except MissingSpatialDimensionError as err:
+        try:
+            obj[var].rio.set_spatial_dims(
+                x_dim=obj.rio.x_dim, y_dim=obj.rio.x_dim, inplace=True
+            )
+            return obj.rio.x_dim, obj.rio.y_dim
+        except MissingSpatialDimensionError:
+            raise err from None
+
+
+def _has_spatial_dims(obj: xarray.Dataset, var: str):
+    """
+    Check to see if the variable in the Dataset has spatial dimensions
+    """
+    try:
+        # pylint: disable=pointless-statement
+        _get_spatial_dims(obj, var)
+    except MissingSpatialDimensionError:
+        return False
+    return True
+
+
 class XRasterBase:
     """This is the base class for the GIS extensions for xarray"""
 
@@ -260,11 +330,7 @@ class XRasterBase:
         if hasattr(self._obj, "data_vars"):
             grid_mappings = set()
             for var in self._obj.data_vars:
-                try:
-                    # pylint: disable=pointless-statement
-                    self._obj[var].rio.x_dim
-                    self._obj[var].rio.y_dim
-                except DimensionError:
+                if not _has_spatial_dims(self._obj, var):
                     continue
                 var_grid_mapping = self._obj[var].encoding.get(
                     "grid_mapping", self._obj[var].attrs.get("grid_mapping")
@@ -296,11 +362,9 @@ class XRasterBase:
         if hasattr(data_obj, "data_vars"):
             for var in data_obj.data_vars:
                 try:
-                    x_dim = data_obj[var].rio.x_dim
-                    y_dim = data_obj[var].rio.y_dim
-                except DimensionError:
+                    x_dim, y_dim = _get_spatial_dims(data_obj, var)
+                except MissingSpatialDimensionError:
                     continue
-
                 # remove grid_mapping from attributes if it exists
                 # and update the grid_mapping in encoding
                 new_attrs = dict(data_obj[var].attrs)
@@ -414,7 +478,7 @@ class XRasterBase:
             raise RuntimeError("crs must be set to estimate UTM CRS.")
 
         # ensure using geographic coordinates
-        if self.crs.is_geographic:
+        if self.crs.is_geographic:  # pylint: disable=no-member
             minx, miny, maxx, maxy = self.bounds(recalc=True)
         else:
             minx, miny, maxx, maxy = self.transform_bounds("EPSG:4326", recalc=True)
@@ -444,11 +508,12 @@ class XRasterBase:
         """
         try:
             # look in grid_mapping
-            return Affine.from_gdal(
-                *np.fromstring(
-                    self._obj.coords[self.grid_mapping].attrs["GeoTransform"], sep=" "
-                )
+            transform = np.fromstring(
+                self._obj.coords[self.grid_mapping].attrs["GeoTransform"], sep=" "
             )
+            # Calling .tolist() to assure the arguments are Python float and JSON serializable
+            return Affine.from_gdal(*transform.tolist())
+
         except KeyError:
             try:
                 return Affine(*self._obj.attrs["transform"][:6])
@@ -513,16 +578,21 @@ class XRasterBase:
             The affine of the :obj:`xarray.Dataset` | :obj:`xarray.DataArray`
         """
         transform = self._cached_transform()
-        if transform and not transform.is_rectilinear:
+        if transform and (
+            not transform.is_rectilinear or _affine_has_rotation(transform)
+        ):
             if recalc:
-                warnings.warn("Non-rectilinear transform found. Unable to recalculate.")
+                warnings.warn(
+                    "Transform that is non-rectilinear or with rotation found. "
+                    "Unable to recalculate."
+                )
             return transform
 
         try:
             src_left, _, _, src_top = self.bounds(recalc=recalc)
             src_resolution_x, src_resolution_y = self.resolution(recalc=recalc)
         except (DimensionMissingCoordinateError, DimensionError):
-            return Affine.identity()
+            return Affine.identity() if transform is None else transform
         return Affine.translation(src_left, src_top) * Affine.scale(
             src_resolution_x, src_resolution_y
         )
@@ -704,13 +774,13 @@ class XRasterBase:
         if x_dim in data_obj.dims:
             data_obj.rio._x_dim = x_dim
         else:
-            raise DimensionError(
+            raise MissingSpatialDimensionError(
                 f"x dimension ({x_dim}) not found.{_get_data_var_message(data_obj)}"
             )
         if y_dim in data_obj.dims:
             data_obj.rio._y_dim = y_dim
         else:
-            raise DimensionError(
+            raise MissingSpatialDimensionError(
                 f"y dimension ({y_dim}) not found.{_get_data_var_message(data_obj)}"
             )
         return data_obj
@@ -720,7 +790,7 @@ class XRasterBase:
         """str: The dimension for the X-axis."""
         if self._x_dim is not None:
             return self._x_dim
-        raise DimensionError(
+        raise MissingSpatialDimensionError(
             "x dimension not found. 'rio.set_spatial_dims()' or "
             "using 'rename()' to change the dimension name to 'x' can address this."
             f"{_get_data_var_message(self._obj)}"
@@ -731,7 +801,7 @@ class XRasterBase:
         """str: The dimension for the Y-axis."""
         if self._y_dim is not None:
             return self._y_dim
-        raise DimensionError(
+        raise MissingSpatialDimensionError(
             "y dimension not found. 'rio.set_spatial_dims()' or "
             "using 'rename()' to change the dimension name to 'y' can address this."
             f"{_get_data_var_message(self._obj)}"
@@ -774,20 +844,20 @@ class XRasterBase:
                 f"{_get_data_var_message(self._obj)}"
             )
         if extra_dims and self._obj.dims != (extra_dims[0], self.y_dim, self.x_dim):
+            dim_info = (extra_dims[0], self.y_dim, self.x_dim)
             raise InvalidDimensionOrder(
-                "Invalid dimension order. Expected order: {0}. "
-                "You can use `DataArray.transpose{0}`"
-                " to reorder your dimensions.".format(
-                    (extra_dims[0], self.y_dim, self.x_dim)
-                )
-                + f"{_get_data_var_message(self._obj)}"
+                f"Invalid dimension order. Expected order: {dim_info}. "
+                f"You can use `DataArray.transpose{dim_info}`"
+                " to reorder your dimensions."
+                f"{_get_data_var_message(self._obj)}"
             )
         if not extra_dims and self._obj.dims != (self.y_dim, self.x_dim):
+            dim_info = (self.y_dim, self.x_dim)
             raise InvalidDimensionOrder(
-                "Invalid dimension order. Expected order: {0}"
-                "You can use `DataArray.transpose{0}` "
-                "to reorder your dimensions.".format((self.y_dim, self.x_dim))
-                + f"{_get_data_var_message(self._obj)}"
+                f"Invalid dimension order. Expected order: {dim_info}. "
+                f"You can use `DataArray.transpose{dim_info}`"
+                " to reorder your dimensions."
+                f"{_get_data_var_message(self._obj)}"
             )
         return extra_dims[0] if extra_dims else None
 
@@ -822,6 +892,9 @@ class XRasterBase:
 
     def resolution(self, recalc=False):
         """
+        Determine if the resolution of the grid.
+        If the transformation has rotation, the sign of the resolution is lost.
+
         Parameters
         ----------
         recalc: bool, optional
@@ -838,9 +911,7 @@ class XRasterBase:
         if (
             not recalc or self.width == 1 or self.height == 1
         ) and transform is not None:
-            resolution_x = transform.a
-            resolution_y = transform.e
-            return resolution_x, resolution_y
+            return _resolution(transform)
 
         # if the coordinates of the spatial dimensions are missing
         # use the cached transform resolution
@@ -849,9 +920,7 @@ class XRasterBase:
         except DimensionMissingCoordinateError:
             if transform is None:
                 raise
-            resolution_x = transform.a
-            resolution_y = transform.e
-            return resolution_x, resolution_y
+            return _resolution(transform)
 
         if self.width == 1 or self.height == 1:
             raise OneDimensionalRaster(
@@ -894,16 +963,20 @@ class XRasterBase:
 
         return left, bottom, right, top
 
-    def isel_window(self, window):
+    def isel_window(self, window, pad=False):
         """
         Use a rasterio.window.Window to select a subset of the data.
 
+        .. versionadded:: 0.6.0 pad
+
         .. warning:: Float indices are converted to integers.
 
         Parameters
         ----------
         window: :class:`rasterio.window.Window`
             The window of the dataset to read.
+        pad: bool, default=False
+            Set to True to expand returned DataArray to dimensions of the window
 
         Returns
         -------
@@ -911,13 +984,13 @@ class XRasterBase:
             The data in the window.
         """
         (row_start, row_stop), (col_start, col_stop) = window.toranges()
-        row_start = math.ceil(row_start) if row_start < 0 else math.floor(row_start)
-        row_stop = math.floor(row_stop) if row_stop < 0 else math.ceil(row_stop)
-        col_start = math.ceil(col_start) if col_start < 0 else math.floor(col_start)
-        col_stop = math.floor(col_stop) if col_stop < 0 else math.ceil(col_stop)
+        row_start = 0 if row_start < 0 else math.floor(row_start)
+        row_stop = 0 if row_stop < 0 else math.ceil(row_stop)
+        col_start = 0 if col_start < 0 else math.floor(col_start)
+        col_stop = 0 if col_stop < 0 else math.ceil(col_stop)
         row_slice = slice(int(row_start), int(row_stop))
         col_slice = slice(int(col_start), int(col_stop))
-        return (
+        array_subset = (
             self._obj.isel({self.y_dim: row_slice, self.x_dim: col_slice})
             .copy()  # this is to prevent sharing coordinates with the original dataset
             .rio.set_spatial_dims(x_dim=self.x_dim, y_dim=self.y_dim, inplace=True)
@@ -934,6 +1007,11 @@ class XRasterBase:
                 inplace=True,
             )
         )
+        if pad:
+            return array_subset.rio.pad_box(
+                *rasterio.windows.bounds(window, self.transform(recalc=True))
+            )
+        return array_subset
 
     def slice_xy(self, minx, miny, maxx, maxy):
         """Slice the array by x,y bounds.
@@ -1002,3 +1080,91 @@ class XRasterBase:
         return rasterio.warp.transform_bounds(
             self.crs, dst_crs, *self.bounds(recalc=recalc), densify_pts=densify_pts
         )
+
+    def write_gcps(self, gcps, gcp_crs, grid_mapping_name=None, inplace=False):
+        """
+        Write the GroundControlPoints to the dataset.
+
+        https://rasterio.readthedocs.io/en/latest/topics/georeferencing.html#ground-control-points
+
+        Parameters
+        ----------
+        gcp: list of :obj:`rasterio.control.GroundControlPoints`
+            The Ground Control Points to integrate to the dataset.
+        gcp_crs: str, :obj:`rasterio.crs.CRS`, or dict
+            Coordinate reference system for the GCPs.
+        grid_mapping_name: str, optional
+            Name of the grid_mapping coordinate to store the GCPs information in.
+            Default is the grid_mapping name of the dataset.
+        inplace: bool, optional
+            If True, it will write to the existing dataset. Default is False.
+
+        Returns
+        -------
+        :obj:`xarray.Dataset` | :obj:`xarray.DataArray`:
+            Modified dataset with Ground Control Points written.
+        """
+        grid_mapping_name = (
+            self.grid_mapping if grid_mapping_name is None else grid_mapping_name
+        )
+        data_obj = self._get_obj(inplace=True)
+
+        data_obj = data_obj.rio.write_crs(
+            gcp_crs, grid_mapping_name=grid_mapping_name, inplace=inplace
+        )
+        geojson_gcps = _convert_gcps_to_geojson(gcps)
+        data_obj.coords[grid_mapping_name].attrs["gcps"] = geojson_gcps
+        return data_obj
+
+    def get_gcps(self):
+        """
+        Get the GroundControlPoints from the dataset.
+
+        https://rasterio.readthedocs.io/en/latest/topics/georeferencing.html#ground-control-points
+
+        Returns
+        -------
+        list of :obj:`rasterio.control.GroundControlPoints` or None
+            The Ground Control Points from the dataset or None if not applicable
+        """
+        try:
+            geojson_gcps = self._obj.coords[self.grid_mapping].attrs["gcps"]
+        except (KeyError, AttributeError):
+            return None
+
+        gcps = [
+            GroundControlPoint(
+                x=gcp["geometry"]["coordinates"][0],
+                y=gcp["geometry"]["coordinates"][1],
+                z=gcp["geometry"]["coordinates"][2],
+                row=gcp["properties"]["row"],
+                col=gcp["properties"]["col"],
+                id=gcp["properties"]["id"],
+                info=gcp["properties"]["info"],
+            )
+            for gcp in geojson_gcps["features"]
+        ]
+        return gcps
+
+
+def _convert_gcps_to_geojson(gcps):
+    """
+    Convert GCPs to geojson.
+
+    Parameters
+    ----------
+    gcps: The list of GroundControlPoint instances.
+
+    Returns
+    -------
+    A FeatureCollection dict.
+    """
+    features = [
+        dict(
+            type="Feature",
+            properties=dict(id=gcp.id, info=gcp.info, row=gcp.row, col=gcp.col),
+            geometry=dict(type="Point", coordinates=[gcp.x, gcp.y, gcp.z]),
+        )
+        for gcp in gcps
+    ]
+    return dict(type="FeatureCollection", features=features)
